function [z]=lfft_gen_lattice(h,M,opts,z_in)
%LFFT_GEN_LATTICE Searches for generating vector z of reconstructing rank-1 lattice.
%
%   [z]=sfft_gen_lattice(h,M,opts,z_in)
%   h:  frequency index set
%   M:  lattice size
%   opts: (optional)
%       opts.Inc [{0} | 1] 
%           - 0:    search for each component starts from 1
%           - 1:    search for each component z_s starts from z_{s-1}+1,
%                   result is an ordered vector [usefull for equalweighted
%                   hyperbolic crosses or l1balls]
%       opts.Display [ {off} | cbc | final]
%           - off:      no information is shown
%           - cbc:      displays the components at the moment you find it
%           - final:    displays the resulting vector, if exists, and the
%                       computational time
%   z_in: (optional) minimal values for z
%
% Copyright (c) 2014 Lutz Kaemmerer, 2015 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.


dim = size(h,2);
% first tests:
if(size(h,1)>M)
    error('You have to sample at least size(h,1) times. So choose M>=size(h,1)!');
end

if(nargin<3)
    opts.Display='off';
    opts.Inc=0;
else
    if(~(isfield(opts,'Inc') && opts.Inc==1))
        opts.Inc=0;
    end
    if(~(isfield(opts,'Display') && strmatch(opts.Display, char('off', 'cbc', 'final'),'exact')>0))
        opts.Display='off';
    end
end

if nargin < 4
    z_in = [];
end

if length(z_in) < dim
    z_in = [z_in, zeros(1, dim-length(z_in))];
end

h=double(h);

ticsearch=tic;
z = [];
for s=1:dim
    hs=unique(h(:,1:s),'rows');
    tmpstart=0;
    switch(opts.Inc)
        case 1
            tmpstart = z(s-1)+1;
    end
    tmpstart = max(tmpstart, z_in(s));
    for l=tmpstart:M-1
       if  length(unique(mod(hs*[z,l]',M)))==size(hs,1)
           z=[z,l];
           break;
       end
    end
    if length(z) ~= s
      return;
    end
    if(strcmp(opts.Display,'cbc')==1)
        fprintf('dimension %d, z_%d=%d\n', [s,s,z(s)])
    end
end
t1=toc(ticsearch);

if(strcmp(opts.Display,'final')==1 || strcmp(opts.Display,'cbc')==1)
    strtoshow=sprintf('generating vector z=[%d',z(1));
    for s=2:length(z)
        strtoshow=[strtoshow,',%d'];
    end
    strtoshow=sprintf([strtoshow,']\n'],z(2:length(z)));
    strtoshow=[strtoshow,sprintf('cbc search time: %d',t1)];
    disp(strtoshow);
end

end
