function [ num_freq_out, err_l2_rel, err_l2_rel_trunc, err_l2_rel_alias, freq_cand_sizes, lattice_sizes, sample_sizes, z, run_times ] = test3_2_sfft_algorithm_singletest( N, algorithm_name, threshold_theta, threshold_theta_basic, niter_r )
%TEST3_SFFT_ALGORITHM_SINGLETEST Performs a single test run of sfft_algorithm on 10d function.
%
%   Input parameters:
%      N - search space parameter N, search space Gamma = [-N,..,N]^d
%      algorithm_name - type of algorithm,
%                       either 'a1r1l' for Algorithm 1
%                       or 'a2r1l' for Algorithm 2
%      sparsity_s - target sparsity of output
%      threshold_theta - relative threshold theta used for
%                        truncation in step 2e, default is 1e-12
%      threshold_theta_basic - relative threshold theta used for
%                              truncation in steps 1 and 2a,
%                              uses the value of threshold_theta as default 
%      niter_r - number of detection iterations r
%
%   Output parameters:
%      num_freq_out - number of frequencies returned by the algorithm
%      err_l2_rel - maximal relative L_2 approximation error
%                   (contains truncation error and aliasing error)
%      err_l2_rel_trunc - maximal relative L_2 truncation error
%      err_l2_rel_alias - maximal relative L_2 aliasing error
%      freq_cand_sizes - array of length d with number of investigated
%                        frequencies in each dimension increment step t
%      lattice_sizes - array of length d with size M of reconstructing
%                      rank-1 lattices used for the sampling
%                      in each dimension increment step t
%      sample_sizes - array of length d with number of samples taken from
%                     the function fct_handle in each dimension increment
%                     step t
%      z - generating vector z of reconstructing rank-1 lattice for
%          frequencies in freq, lattice_sizes(end) contains the
%          corresponding rank-1 lattice size M
%      run_times - array of length 5 with runtimes measured for various
%                  parts of the algorithm,
%                  run_times(1): steps 1 and 2a,
%                  run_times(2): building of reconstructing rank-1 lattices
%                  run_times(3): sampling step 2c
%                  run_times(4): inverse rank-1 lattice FFT step 2d
%                  run_times(5): total runtime of the algorithm
%
% Copyright (c) 2015 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

sparsity_s = intmax;
sparsity_s_local = intmax;
niter_r_basic = niter_r;

[ freq_out, fhat_out, freq_cand_sizes, lattice_sizes, sample_sizes, z, run_times ] = sfft_algorithm( @bspline_test_10d, 10, N, algorithm_name, sparsity_s, sparsity_s_local, threshold_theta, threshold_theta_basic, niter_r, niter_r_basic );

fhat = bspline_test_10d_fouriercoeff(freq_out);

num_freq_out = size(freq_out,1);

%norm_fct_square = 3 + 2*bspline_o2_hat(zeros(1,3))*bspline_o4_hat(zeros(1,4)) + 2*bspline_o2_hat(zeros(1,3))*bspline_o6_hat(zeros(1,3)) + 2*bspline_o4_hat(zeros(1,4))*bspline_o6_hat(zeros(1,3));
norm_fct_square = 3.86052137015856367;
fprintf('norm fct: %.6e, ', sqrt(norm_fct_square));
err_l2_rel_trunc = sqrt(norm_fct_square - norm(fhat,2)^2) / sqrt(norm_fct_square);
fprintf('rel. truncation err.: %.3e, ', err_l2_rel_trunc);
err_l2_rel_alias = norm(fhat_out-fhat,2) / sqrt(norm_fct_square);
fprintf('rel. aliasing err.: %.3e\n', err_l2_rel_alias);

err_l2_rel = sqrt( norm_fct_square - norm(fhat,2)^2 + norm(fhat_out-fhat,2)^2 ) / sqrt(norm_fct_square);
fprintf('relative L_2 approximation error: %3e\n', err_l2_rel);

end

